<?php

class CF_Filters {

  public static function init() {
    add_filter('acf/settings/save_json', ['CF_Filters', 'acf_json_save_point'], 1);
    add_filter('acf/settings/load_json', ['CF_Filters', 'acf_json_load_point'], 20);
    add_filter('script_loader_tag', ['CF_Filters', 'add_asyncdefer_attribute'], 10, 2);
    add_filter('template_include', ['CF_Wrapping', 'wrap'], 109);
    add_filter('user_contactmethods', ['CF_Filters', 'update_contact_methods']);
    add_filter('posts_search', ['CF_Filters', 'search_by_title_only'], 500, 2 );
    add_filter('pre_get_posts', ['CF_Filters', 'searchfilter']);
    add_filter('acf-flexible-content-preview.images_path', ['CF_Filters', 'acf_preview_path']);
    add_filter('excerpt_more', ['CF_Filters','excerpt_more']);
    add_filter('wp_lazy_loading_enabled', '__return_false');
    add_filter('option_show_avatars', '__return_false');
    add_filter('wp_nav_menu_objects', ['CF_Filters', 'nav_menu_objects'], 10, 2);
    add_filter('walker_nav_menu_start_el', ['CF_Filters', 'megamenu'], 15, 4);
  }

  public static function acf_json_save_point($path) {
    $path = CF_ACF_JSON_DIR;
    return $path;
  }

  public static function acf_json_load_point($paths) {
    unset($paths[0]);
    $paths[] = CF_ACF_JSON_DIR;
    return $paths;
  }

  public static function add_asyncdefer_attribute($tag, $handle) {
    if(!is_admin()) :
      if (strpos($handle, 'async') !== false || strpos($handle, 'smush-lazy-load') !== false) 
        return str_replace( '<script ', '<script async ', $tag );

      if (strpos($handle, 'defer') !== false)
        return str_replace( '<script ', '<script defer ', $tag );
    endif;

    return $tag;
  }

  public static function update_contact_methods($contactmethods) {
    unset($contactmethods['facebook']);
    unset($contactmethods['instagram']);
    unset($contactmethods['linkedin']);
    unset($contactmethods['myspace']);
    unset($contactmethods['pinterest']);
    unset($contactmethods['soundcloud']);
    unset($contactmethods['tumblr']);
    unset($contactmethods['twitter']);
    unset($contactmethods['youtube']);
    unset($contactmethods['wikipedia']);
    return $contactmethods;
  }

  public static function search_by_title_only( $search, $wp_query) {
    global $wpdb;
  
    if(!$wp_query->query_vars)
      return $search;
  
    if (empty($search))
      return $search;
  
    $q = $wp_query->query_vars;
    $n = !empty($q['exact']) ? '' : '%';
    $search =
    $searchand = '';
  
    foreach ((array)$q['search_terms'] as $term) {
      $term = esc_sql(like_escape($term));
      $search .= "{$searchand}($wpdb->posts.post_title LIKE '{$n}{$term}{$n}')";
      $searchand = ' AND ';
    }
    if ( ! empty( $search ) ) {
      $search = " AND ({$search}) ";
      if ( ! is_user_logged_in() )
          $search .= " AND ($wpdb->posts.post_password = '') ";
    }
    return $search;
  }

  public static function searchfilter($query) {
    if ($query->is_search && !is_admin()) :
      $query->set('orderby', 'type');
      $query->set('post_status', 'publish');
      $query->set('posts_per_page', 24);
    endif;
  
    return $query;
  }

  public static function acf_preview_path() {
    return 'resources/preview-acf';
  }

  public static function excerpt_more() {
    return '...';
  }

  public static function nav_menu_objects($items, $args) {
    if ($args->theme_location === 'primary_navigation') :
      
      foreach($items as $item) :
        if ( get_field( 'submenu', $item->ID ) ) :
          $item->classes[] = 'menu-item-has-children ps-s';
        endif;
      endforeach;
    endif;
    
    return $items;
  }

  public static function megamenu( $item_output, $item, $depth, $args ) {
    if ($args->theme_location === 'primary_navigation') :
      if ( $submenus = get_field( 'submenu', $item->ID ) ) :
      ob_start(); 
      ?>
        <div class="megamenu of-h p-d-md-20">
          <div class="container mw-d-md-100">
            <div class="mx-u-md-n10 mx-d-md-n20 d-f f-d-md-w">
              <?php foreach ($submenus as $submenu) : ?>
                <div class="megamenu__item mb-15 px-20">
                  <p class="mb-20 tc-d fw-sb fs-16"><?php echo $submenu['menu']->name; ?></p>
                  <?php
                    wp_nav_menu([
                      'menu' => $submenu['menu']->term_id,
                      'container' => null,
                      'menu_id' => null,
                      'menu_class' => 'sub-menu'
                    ]);
                  ?>
                </div>
              <?php endforeach; ?>
            </div>
  
             <?php
              $special_link = get_field( 'special_link', $item->ID );
              if ( $special_link['url'] && $special_link['link'] ) :
            ?>
              <a href="<?php echo $special_link['url']; ?>" class="d-u-lg-f d-d-md-n p-u-lg-5 ai-u-lg-c jc-u-lg-c br-u-lg-5 tc-u-lg-w fw-u-lg-n fs-u-lg-13 bg-u-lg-p bg-u-lg-h-b w-u-lg-100 mt-u-lg-20">
                <?php 
                  if ( $special_link['icon'] ) :
                    echo wp_get_attachment_image( $special_link['icon'], 'full', true, ['class' => 'mt-u-lg-n10']);  
                  endif;
                  echo $special_link['link'];
                ?>
              </a>
            <?php endif; ?>
          </div>
        </div>
      <?php
      $item_output .= ob_get_clean();
      endif;
    endif;

    return $item_output;
  }
}