<?php

class CF_Helpers_Casino {

  protected static function get_external_data($post_id, $key, $value) {

    if ( empty ( $value ) ) return $value;

    $taxonomies_fields = [
      'owners'             => 'owner',
      'badge'              => 'badge',
      'casino_types'       => 'casino-type',
      'licenses'           => 'license',
      'languages'          => 'language',
      'platforms'          => 'platform',
      'software_providers' => 'provider',
      'game_types'         => 'game-type',
      'currencies'         => 'currency',
      'crypto_currencies'  => 'crypto-currency',
      'deposit_methods'    => 'payment',
      'withdraw_methods'   => 'payment',
    ];

    $true_false_fields = [];

    if ( array_key_exists($key, $taxonomies_fields) ) {
      $terms = CF_Helpers_Taxonomy::get_taxonomy($value);

      if ( $key === 'badge' ) {
        $value = CF_Helpers_Taxonomy::get_taxonomy_fields($terms, ['text_color', 'color', 'logo'], $taxonomies_fields[$key]);
        $value = end($value);
      } elseif( $key === 'licenses' || $key === 'languages' ) {
        $value = CF_Helpers_Taxonomy::get_taxonomy_fields($terms, ['linked', 'logo'], $taxonomies_fields[$key]);
      } elseif( $key === 'currencies' ) {
        $value = CF_Helpers_Taxonomy::get_taxonomy_fields($terms, ['linked', 'symbol'], $taxonomies_fields[$key]);
      } elseif( $key === 'deposit_methods' || $key === 'software_providers' ) {
        $value = CF_Helpers_Taxonomy::get_taxonomy_fields($terms, ['linked', 'logo', 'color'], $taxonomies_fields[$key]);
      } elseif( $key === 'casino_type') {
        $value = CF_Helpers_Taxonomy::get_taxonomy_fields($terms, ['logo'], $taxonomies_fields[$key]);
      } elseif ( $key === 'crypto_currencies' || $key === 'game_types' || $key === 'owners' || $key === 'platforms' ) {
        $value = CF_Helpers_Taxonomy::get_taxonomy_fields($terms, ['linked'], $taxonomies_fields[$key]);
      } else {
        $value = $terms;
      }
    } elseif ( in_array( $key, $true_false_fields ) ) {
      $value = $value ? __('Yes', 'elegance') : __('No', 'elegance');
    } elseif ( $key === 'terms_url' ) {
      $value = ' | <a target="_blank" href="'. $value .'" class="tc-in">'. __('T&C apply', 'elegance') .'</a>';
    } elseif ( $key ===  'advantages' || $key ===  'disadvantages' ) {
      $value = CF_Helpers_Common::acf_repeater($post_id, $key, ['item']);
    } elseif ( $key ===  'live_games_list' ) {
      $value = CF_Helpers_Common::acf_repeater($post_id, $key, ['title', 'games']);
    } elseif ( $key ===  'payments_matrix' ) {
      $value = CF_Helpers_Common::acf_repeater($post_id, $key, ['payment_method', 'deposit', 'withdrawal', 'time']);
    } elseif ( $key ===  'crypto_wallet' ) {
      $value = CF_Helpers_Common::acf_repeater($post_id, $key, ['crypto_currency', 'casino', 'slots', 'roulette',  'baccarat', 'poker', 'blackjack', 'live_casino', 'esports', 'sports']);
    } elseif ( $key ===  'website' ) {
      $parse = parse_url($value);
      $value = $parse['host'];
    } elseif ( $key ===  'restricted' ) {
      $countries = [];
      foreach ($value as $post_id) {
        $countries[] = get_post_meta( $post_id, 'country_code', true );
      }
      $value = $countries;
    } elseif ( $key === 'wagering' ) {
      $value = 'x' . $value;
    }

    return $value;
  }

  protected static function get_geo_default_bonus($post_id) {
    $bonuses = CF_Helpers_Common::acf_group($post_id, 'default_bonuses', ['welcome_bonus', 'first_deposit_bonus', 'bonus_amount', 'bonus_percentage', 'free_spins', 'wagering', 'terms', 'terms_url']);

    return $bonuses;
  }

  protected static function get_geo_bonus($post_id) {
    $bonuses = CF_Helpers_Common::acf_repeater($post_id, 'geo_targeted_bonuses', ['country', 'welcome_bonus', 'first_deposit_bonus', 'bonus_amount', 'bonus_percentage', 'free_spins', 'wagering', 'terms', 'terms_url']);

    if ( ! $bonuses ) return;

    $geo_bonus = null;
    $user_country = (int)CF_Helpers_Common::get_session('country_id');

    foreach($bonuses as $bonus) :
      if ( (int) $bonus['country'] === $user_country) :
        $geo_bonus = $bonus;
        break;
      endif;
    endforeach;

    return $geo_bonus;
  }

  public static function the_permalink($post_id, $echo = true) {
    // if (CF_Helpers_Common::disable_single_page($post_id)) :
    //   $url = self::get_casino_fields($post_id, 'affiliate_url');
    // else:
      $url = get_the_permalink($post_id);
    // endif;

    if ( ! $url ) return null;

    if ($echo) echo $url;
    else return $url;
  }

  public static function get_casino_bonus($post_id) {
    $bonuses = self::get_geo_bonus($post_id) ?: self::get_geo_default_bonus($post_id);

    foreach ($bonuses as $key => $value) {
      $bonuses[$key] = self::get_external_data($post_id, $key, $value);
    }

    return $bonuses;
  }

  public static function get_casino_fields($post_id, $fields = null) {

    $groups = [
      'base' => [
        'is_restricted',
        'rating',
        'color',
        'affiliate_url',
        'geobonuses',
        'badge',
      ],
      'card' => [
        'live_games_amount',
        'deposit_methods',
        'software_providers',
        'minimum_deposit',
        'casino_types'
      ],
      'card_bonus' => [
        'is_restricted',
        'rating',
        'color',
        'affiliate_url',
        'badge',
      ],
      'list_bonus' => [
        'is_restricted',
        'rating',
        'color',
        'affiliate_url',
        'badge',
        'advantages'
      ],
      'list' => [
        'advantages'
      ],
      'bonus' => [
        'is_restricted',
        'rating',
        'color',
        'affiliate_url',
        'badge',
        'advantages',
        'languages',
        'currencies',
        'deposit_methods',
        'licenses',
        'introduction',
        'position_on_list',
        'gallery'
      ],
      'single' => [
        'position_on_list',
        'advantages',
        'languages',
        'currencies',
        'crypto_currencies',
        'introduction',
        'deposit_methods',
        'licenses',
        'rating_interface',
        'rating_reputation',
        'rating_bonus_quality',
        'rating_game_selection',
        'rating_customer_support',
        'rating_payments',
        'rewards_program',
        'cashback',
        'bonus_code',
        'exclusive_bonus',
        'tournaments',
        'bonuses_occasions',
        'live_casino',
        'sportsbooks',
        'mobile_casino',
        'crypto',
        'vip_program',
        'pay_n_play',
        'gamification',
        'online_chat',
        'email_support',
        'slots_amount',
        'live_games_amount',
        'minimum_deposit',
        'minimum_withdrawal',
        'withdrawal_limitations',
        'software_providers',
        'website',
        'email',
        'phone',
        'year_founded',
        'listed_us',
        'platforms',
        'owners',
        'gallery',
        'disadvantages',
        'news',
        'bonuses',
        'live_games_list',
        'payments_matrix',
        'crypto_wallet',
        'restricted'
      ]
    ];

    if ( ! $fields ) :
      return self::get_casino_fields($post_id, ['base', 'single']);
    elseif ( is_string($fields) ) :
      if ( array_key_exists($fields, $groups) ) :
        $value = self::get_casino_fields($post_id, $groups[$fields]);
      elseif ($fields === 'geobonuses') :
        $value = self::get_casino_bonus($post_id);
      elseif ($fields === 'is_restricted') :
        $value = CF_Helpers_Common::is_restricted($post_id);
      else :
        $value = self::get_external_data($post_id, $fields, get_post_meta( $post_id, $fields, true ));
      endif;
      return $value;
    elseif ( is_array( $fields ) ) :
      $array['post_id'] = $post_id;
      foreach( $fields as $key ) :
        if ( array_key_exists($key, $groups) ) :
          $array = array_merge($array, self::get_casino_fields($post_id, $groups[$key]));
        elseif ( $key === 'geobonuses' ) :
          $array = array_merge($array, self::get_casino_fields($post_id, $key) );
        else :
          $array[$key] = self::get_casino_fields($post_id, $key);
        endif;
      endforeach;

      return $array;
    endif;

    return null;
  }
}
