<?php

class CF_Helpers_Common {

  /*
   * Get Casino Feed Sesstion
   */
  public static function get_session($keys = null) {
    if (empty($keys) && isset($_SESSION['casinofeed'])) :
      return $_SESSION['casinofeed'];
    elseif(is_string($keys) && isset($_SESSION['casinofeed'][$keys])) :
      return $_SESSION['casinofeed'][$keys];
    elseif(is_array($keys)) :
      $results = [];
      foreach($keys as $key) :
        $results[$key] = self::get_session($key);
      endforeach;

      return $results;
    endif;

    return null;
  }

  /*
   * Set Casino Feed Sesstion
   */
  public static function set_session($key = null, $value = '') {
    $_SESSION['casinofeed'][$key] = $value;
  }

  /*
   * Checking for Disabled single page
   */
  public static function disable_single_page($post_id) {
    return get_post_meta( $post_id, 'disable_single', true );
  }

  /*
   * Get Post Type Currency
   */
  public static function get_currency($post_type = 'post') {
    return [
      'symbol' => get_option( "options_${post_type}_symbol"),
      'position' => get_option("options_${post_type}_symbol_position")
    ];
  }

  public static function currency_position($currency, $value) {
    if (empty($value))
      return '';

    if ($currency['position'] === 'front') :
      return $currency['symbol'] . $value;
    endif;

    if ($currency['position'] === 'rear') :
      return $value . $currency['symbol'];
    endif;
  }

   /*
   * Get Post Type Currency
   */
  public static function set_currency($post_type = 'post', $value = '') {
    $currency = self::get_currency($post_type);
    return self::currency_position($currency, $value);
  }

  /*
   * Get Post Currency
   */
  public static function currency($post_id = null) {
    $post_type = get_post_type($post_id);
    return self::get_currency($post_type);
  }

  /*
   * Get Post Currency
   */
  public static function set_currency_symbol($post_id, $value) {
    $currency = self::currency($post_id);
    return self::currency_position($currency, $value);
  }

  /*
   * Checking restricted post
   */
  public static function is_restricted($post_id) {
    $restricted = get_post_meta( $post_id,'restricted', true);
    if ( !$restricted ) return false;

    $customer = CF_Helpers_Common::get_session(['geo_iso_code', 'country_id']);
    if ( $customer['geo_iso_code'] === 'US' ) {
      $us_page_id = CF_Helpers_Country::get_country_id_by_code('US', false);

      return in_array($customer['country_id'], $restricted) || in_array($us_page_id, $restricted);
    }

    return in_array($customer['country_id'], $restricted);
  }

  /*
   * ACF Get Repeater
   */
  public static function acf_repeater($post_id, $field, $sub_fields) {
    $repeater = get_post_meta( $post_id, $field, true );

    if ( ! $repeater ) return false;

    $rows = [];
    for ( $i = 0; $i < $repeater; $i++) {
      foreach ($sub_fields as $sub_field) {
        $rows[$i][$sub_field] = get_post_meta( $post_id, $field . '_' . $i . '_' . $sub_field, true );
      }
    }

    return $rows;
  }

  /*
   * ACF Get Group
   */
  public static function acf_group($post_id, $field, $sub_fields) {
    $group = [];
    foreach ($sub_fields as $sub_field) {
      $group[$sub_field] = get_post_meta( $post_id, $field . '_' . $sub_field, true );
    }

    return $group;
  }

  /*
   * Display Title
   */
  public static function display_title($title, $classes = null) {
    if (empty($title))
      return;

    if ($title['color'] !== 0);
      $classes .= ' ' . $title['color'];

    $tag = $title['tag'];
    $title = $title['title'];
    $attr = '';

    if ($classes) $attr = ' class="' . $classes . '"';

    echo "<${tag}${attr}>${title}</${tag}>";
  }

  /*
   * ACF Get Button
   */
  public static function button($cta) {
    if (!$cta || (isset($cta['cta_options']) && $cta['cta_options'] === '0')) return false;

    if (!isset($cta['cta_options'])) $cta['cta_options'] = '4';

    $btn['label'] = isset($cta['label']) ? $cta['label'] : false;
    $btn['classes'] =  '';
    // $btn['disabled'] = false;

    if (isset($cta['type'])) :
      if ($cta['type'] === '1') :
        $btn['classes'] .= $cta['text_color'] . ' btn--' . $cta['btn_color'];
      elseif ($cta['type'] === '2') :
        $btn['classes'] .= $cta['text_color'] . ' btn-g--' . $cta['btn_color'];
      endif;
    endif;

    if ($cta['cta_options'] === '1') :
      $btn['action'] = $cta['page_link'];
    elseif ($cta['cta_options'] === '2') :
      $btn['action'] = $cta['enter_link'];
    elseif ($cta['cta_options'] === '3') :
      $btn['action'] = $cta['anchor_id'];
      $btn['classes'] .= ' js-anchor';
    endif;

    return $btn;
  }

  /*
   * ACF Get Button
   */
  public static function layout($layout) {
    return str_replace('0', '', implode(' ', $layout));
  }

  /*
  * Display Pagination
  */
  public static function pagination($wp_query) {
    $big = 999999999;
    $paged = get_query_var('paged');
    $max_page = $wp_query->max_num_pages;

    $links = paginate_links([
      'base'       => str_replace($big, '%#%', esc_url(get_pagenum_link($big))),
      'format'     => '?paged=%#%',
      'current'    => max(1, $paged),
      'total'      => $max_page,
      'mid_size'   => 1,
      'prev_text'  => '<svg width="16" height="16" viewBox="0 0 16 16" class="va-m" fill="none" xmlns="http://www.w3.org/2000/svg"><path fill-rule="evenodd" clip-rule="evenodd" d="M0.887193 8.70705C0.496668 8.31653 0.496668 7.68336 0.887193 7.29284L7.25115 0.92888C7.64168 0.538355 8.27484 0.538355 8.66537 0.92888C9.05589 1.3194 9.05589 1.95257 8.66537 2.34309L4.00851 6.99995H14.9699C15.5222 6.99995 15.9699 7.44766 15.9699 7.99995C15.9699 8.55223 15.5222 8.99995 14.9699 8.99995H4.00851L8.66537 13.6568C9.05589 14.0473 9.05589 14.6805 8.66537 15.071C8.27484 15.4615 7.64168 15.4615 7.25115 15.071L0.887193 8.70705Z" fill="#232222"/></svg>
      ',
      'next_text'  => '<svg width="15" height="16" viewBox="0 0 15 16" class="va-m" fill="none" xmlns="http://www.w3.org/2000/svg"><path fill-rule="evenodd" clip-rule="evenodd" d="M8.34327 0.92888L14.7072 7.29284C15.0978 7.68336 15.0978 8.31653 14.7072 8.70705L8.34327 15.071C7.95274 15.4615 7.31958 15.4615 6.92905 15.071C6.53853 14.6805 6.53853 14.0473 6.92905 13.6568L11.5859 8.99995H1.51611C0.963829 8.99995 0.516113 8.55223 0.516113 7.99995C0.516113 7.44766 0.963829 6.99995 1.51611 6.99995H11.5859L6.92905 2.34309C6.53853 1.95257 6.53853 1.3194 6.92905 0.92888C7.31958 0.538355 7.95274 0.538355 8.34327 0.92888Z" fill="#232222"/></svg>
      ',
    ]);

    $links = str_replace('<a class="prev', '<a rel="prev"  class="prev', $links);
    $links = str_replace('<a class="next', '<a rel="next"  class="next', $links);

    echo $links;
  }

  /*
  * Get SVG Code
  */
  public static function get_svg($svg) {
    return file_get_contents( $svg );
  }
}
