<?php

class CF_Helpers_Post_Type {

  protected static function get_external_data($post_id, $key, $value) {

    if ( empty ( $value ) ) return $value;

    $taxonomies_fields = [
      'type'     => 'payment-category',
      'features' => get_post_type() . '-feature',
      'badge'    => 'badge',
    ];

    if ( array_key_exists($key, $taxonomies_fields) ) {
      $terms = CF_Helpers_Taxonomy::get_taxonomy($value);
      if ( $key === 'badge' ) {
        $value = CF_Helpers_Taxonomy::get_taxonomy_fields($terms, ['text_color', 'color', 'logo'], $taxonomies_fields[$key]);
        $value = end($value);
      } else {
        $value = $terms;
      }
    } elseif ( $key ===  'website' ) {
      $parse = parse_url($value);
      $value = '<a href="'. $value .'" target="_blank" class="td-u tc-h-p">'.  $parse['host'] . '</a>';
    } elseif ( $key === 'gambling_options' ) {
      $value = CF_Helpers_Common::acf_repeater($post_id, $key, ['icon', 'url', 'text']);
    }

    return $value;
  }

  public static function get_post_type_fields($post_id, $fields) {
    $groups = [
      'payment' => [
        'rating',
        'fees',
        'processing_time',
        'type',
        'introduction',
        'number_casinos'
      ],
      'provider' => [
        'rating',
        'founded',
        'badge',
        'number_of_games',
        'country',
        'introduction',
        'features',
        'number_casinos'
      ],
      'license' => [
        'introduction',
        'rating',
        'website',
        'country',
        'number_casinos'
      ],
      'crypto-currency' => [
        'introduction',
        'code',
        'gambling_options',
        'number_of_casinos'
      ],
      'platform' => [
        'rating',
        'introduction',
        'number_casinos',
        'licenses',
        'providers',
        'payments',
        'features'
      ],
      'currency'   => [ 'introduction' ],
      'owner'      => [ 'introduction' ],
      'countries'  => [ 'introduction' ],
    ];

    if ( ! $fields ) :
      return null;
    elseif ( is_string($fields) ) :
      if ( array_key_exists($fields, $groups) ) :
        $value = self::get_post_type_fields($post_id, $groups[$fields]);
      elseif ($fields === 'number_casinos'):
        $value = self::get_number_casinos($post_id, get_post_type());
      elseif ($fields === 'is_restricted') :
        $value = CF_Helpers_Common::is_restricted($post_id);
      else :
        $value = self::get_external_data($post_id, $fields, get_post_meta( $post_id, $fields, true ));
      endif;
      return $value;
    elseif ( is_array( $fields ) ) :
      $array['post_id'] = $post_id;
      foreach( $fields as $key ) :
        if ( array_key_exists($key, $groups) ) :
          $array = array_merge($array, self::get_post_type_fields($post_id, $groups[$key]));
        else :
          $array[$key] = self::get_post_type_fields($post_id, $key);
        endif;
      endforeach;

      return $array;
    endif;

    return null;
  }

  protected static function get_linked_terms($post_id, $taxonomy) {
    $terms = get_terms([
      'taxonomy' => $taxonomy,
      'post_type' => 'post',
      'hide_empty' => false,
      'meta_query' => [
        [
          'key' => 'linked',
          'value' => $post_id,
          'compare' => 'LIKE'
        ]
      ]
    ]);

    if ( $terms ) return $terms[0];

    return;
  }

  protected static function get_post_count_by_country($country) {
    $query_args = CF_Casinos_Args::list_casinos_args([
      'list_type'          => '3',
      'results_per_page'   => '-1',
      'number_of_results'  => '-1',
      'order_by'           => 'date',
      'allowed_in'         => '2',
      'specific_countries' => [$country],
    ]);
    $casinos = new WP_Query($query_args);
    $count = $casinos->post_count;
    wp_reset_query();
    return $count;
  }

  protected static function get_post_count_by_provider($provider) {
    $query_args = CF_Casinos_Args::list_casinos_args([
      'list_type'                  => '3',
      'results_per_page'           => '-1',
      'number_of_results'          => '-1',
      'order_by'                   => 'date',
      'software_providers_listing' => [$provider],
      'casino_status_listing'      => [658]
    ]);
    $casinos = new WP_Query($query_args);
    $count = $casinos->post_count;
    wp_reset_query();
    return $count;
  }

  public static function get_number_casinos($post_id, $taxonomy) {
    if ( $taxonomy === 'country' )
      return self::get_post_count_by_country($post_id);
    elseif ($term = self::get_linked_terms($post_id, $taxonomy) ) {
      if ( $taxonomy === 'provider' ) 
        return self::get_post_count_by_provider($term->term_id);
      
      return $term->count;
    }
    
    return 0;
  }

  public static function get_linked_term_id($post_id, $taxonomy) {
    if ( $term = self::get_linked_terms($post_id, $taxonomy) ) 
      return $term->term_id;
    
    return;
  }
}
