<?php

class CF_Helpers_Slot {

  protected static function get_external_data_slot($post_id, $key, $value) {
    if ( empty ( $value ) ) return $value;
    
    $taxonomies_fields = [
      'providers'       => 'provider',
      'platforms'       => 'platform',
      'themes'          => 'theme',
      'slot_type'       => 'slot-type',
      'features'        => 'feature',
      'available'       => 'devices',
      'themes'          => 'theme',
      'special_symbols' => 'special-symbols',
    ];

    $true_false_fields = [];

    if ( array_key_exists($key, $taxonomies_fields) ) {
      $terms = CF_Helpers_Taxonomy::get_taxonomy($value);

      if ( $key === 'providers' || $key === 'slot_type' ) {
        $value = CF_Helpers_Taxonomy::get_taxonomy_fields($terms, ['linked'], $taxonomies_fields[$key]);
      } else {
        $value = $terms;
      }
    } elseif ( $key === 'minimum_bet' ||  $key === 'maximum_bet' || $key === 'maximum_win' ) {
      $value = CF_Helpers_Common::set_currency_symbol($post_id, $value);
    } elseif ( $key ===  'advantages' || $key ===  'disadvantages' ) {
      $value = CF_Helpers_Common::acf_repeater($post_id, $key, ['item']);
    }

    return $value;
  }

  public static function get_slot_fields($post_id, $fields = null) {
    $groups = [
      'base' => [
        'rating',
        'play_for_free',
      ],
      'single' => [
        'introduction',
        'providers',
        'rtp',
        'volatility',
        'progressive',
        'slot_type',
        'rows',
        'reels',
        'paylines',
        'slot_layout',
        'minimum_bet',
        'maximum_bet',
        'maximum_win',
        'features',
        'available',
        'date_launched',
        'themes',
        'jackpot',
        'gamification',
        'special_symbols',
        'gallery',
        'advantages',
        'disadvantages',
        'casinos'
      ],
      'list' => [
        'rating',
        'providers',
        'rtp',
        'volatility',
        'paylines',
        'reels',
        'play_real'
      ],
      'quick_facts' => [
        'providers',
        'platform',
        'free_spins',
        'reels',
        'volatility',
        'released',
        'rols',
        'rtp',
        'themes',
        'autoplay',
        'paylines'
      ]
    ];

    if ( ! $fields ) :
      return self::get_slot_fields($post_id, [ 'base', 'single' ] );
    elseif( is_string( $fields ) ) :
      if ( array_key_exists($fields, $groups) ) :
        $value = self::get_slot_fields($post_id, $groups[$fields]);
      else:
        $value = self::get_external_data_slot($post_id, $fields, get_post_meta( $post_id, $fields, true ));
      endif;

      return $value;
    elseif ( is_array( $fields ) ) :
      $array = [];
      foreach( $fields as $key ) :
        if ( array_key_exists($key, $groups) ) :
          $array = array_merge($array, self::get_slot_fields($post_id, $groups[$key]));
        elseif ($key === 'play_real') :
          $array[$key] = self::get_play_casino($post_id);
        elseif ($key === 'casinos') :
          $array = array_merge($array, self::get_slot_casinos($post_id));
        else :
          $array[$key] = self::get_slot_fields($post_id, $key);
        endif;
      endforeach;

      return $array;
    endif;

    return null;
  }

  public static function get_play_casino($post_id) {
    $params = [
      'results_per_page'           => '1',
      'number_of_results'          => '1',
      'order_by'                   => 'meta_value_num',
      'allowed_in'                 => '0',
      'list_type'                  => '3',
      'software_providers_listing' => get_post_meta( $post_id, 'providers', true )
    ];
    $casino = null;
    
    $query_args = CF_Casinos_Args::list_casinos_args($params);
    $casinos = new WP_Query($query_args);
    $casino = wp_list_pluck( $casinos->posts, 'ID' );
    wp_reset_query();

    return CF_Helpers_Casino::get_casino_fields($casino[0], 'affiliate_url');
  }

  public static function get_slot_casinos($post_id) {
    $params = [
      'results_per_page'           => '6',
      'number_of_results'          => '6',
      'order_by'                   => 'meta_value_num',
      'allowed_in'                 => '1',
      'list_type'                  => '3',
      'software_providers_listing' => get_post_meta( $post_id, 'providers', true ),
      'casino_status_listing'      => [658]
    ];
    $casinos = [];
    $casino = null;
    
    $query_args = CF_Casinos_Args::list_casinos_args($params);
    $casinos = new WP_Query($query_args);
    $casinos = wp_list_pluck( $casinos->posts, 'ID' );
    wp_reset_query();
      
    if ($casinos) :
      $casino = self::get_slot_casino($casinos[0]);
    endif;

    return [
      'casinos' => $casinos,
      'casino' => $casino
    ];
  }

  public static function get_slot_casino($casino_id) {
    if (!$casino_id) return;

    $casino = CF_Helpers_Casino::get_casino_fields($casino_id, ['base', 'card']);
    $casino['permalink'] = CF_Helpers_Casino::the_permalink($casino_id, false);
    $casino['featured_image'] = get_the_post_thumbnail($casino_id, 'CF_casino');

    return $casino;
  }
}