<?php 

class CF_Bonuses_Args {

  private static $post_type = 'bonus';
  private static $subfix = '_listing';

  protected static function list_casino_taxonomies($params) {
    $tax_query = [];
    $taxonomies = [
      'bonus-type' => 'bonus_types' . self::$subfix,
      'bonus-occasion' => 'bonus_occasion' . self::$subfix,
    ];

    foreach($taxonomies as $term => $taxonomy) :
      if (isset($params[$taxonomy]) && !empty($params[$taxonomy])) :
        if ( is_array( $params[$taxonomy] ) ) :
          foreach ($params[$taxonomy] as $term_id) {
            $tax_query[] = [
              'taxonomy' => $term,
              'field' => 'term_id',
              'terms' => $term_id,
            ];
          }
        else :
          $tax_query[] = [
            'taxonomy' => $term,
            'field' => 'term_id',
            'terms' => $params[$taxonomy],
          ];
        endif;
      endif;
    endforeach;

    return $tax_query;
  }

  public static function list_bonuses_args($params) {
    $args = [
      'post_type'   => self::$post_type,
      'post_status' => 'publish',
    ];
    $page_posts = (int) $params['results_per_page'];
    $total_posts = (int) $params['number_of_results'];
    $meta_query = isset( $params['meta_query'] ) ? $params['meta_query'] : [];
    $tax_query = isset( $params['tax_query'] ) ? $params['tax_query'] : [];
    $post_in =  isset( $params['found_posts'] ) ?  $params['found_posts'] : [];

    if ($params['list_type'] === '2') {
      $post_in = $params['bonuses'];
      $args['orderby'] = $params['order_by_manually'];
    } else {
      $args['orderby'] = $params['order_by']; 
    }

    if ( $args['orderby'] === 'post__in' ) {
      $args['orderby'] = 'post__in';
    } elseif ( $args['orderby'] !== 'date' ) {
      $args['meta_key'] = $args['orderby'];
      $args['orderby'] = 'meta_value_num date';
      $args['order'] = 'DESC';
    }

    if (isset($params['offset'])) 
      $args['offset'] = $params['offset'];
    
    if (isset($params['s'])) 
      $args['s'] = $params['s'];


    if ($params['list_type'] === '3') {
      if ($taxonomies = self::list_casino_taxonomies($params)) {
        $tax_query = array_merge($tax_query, $taxonomies);
      }

      // if ( isset ( $params['rating'] ) && $params['rating'] != '0' ) {
      //   $meta_query[] = [
      //     'key' => 'rating',
      //     'value' => $params['rating'],
      //     'compare' => '>=',
      //     'type' => 'NUMERIC',
      //   ];
      // }

      if ( isset ( $params['bonus_amount'] ) && $params['bonus_amount'] != '0' ) {
        $meta_query[] = [
          'key' => 'bonus_amount',
          'value' => $params['bonus_amount'],
          'compare' => '>=',
          'type' => 'NUMERIC',
        ];
      }

      if ( isset ( $params['free_spins'] ) && $params['free_spins'] != '0' ) {
        $meta_query[] = [
          'key' => 'free_spins',
          'value' => $params['free_spins'],
          'compare' => '>=',
          'type' => 'NUMERIC',
        ];
      }

      if ( isset ( $params['bonus_percentage'] ) && $params['bonus_percentage'] != '0' ) {
        $meta_query[] = [
          'key' => 'bonus_percentage',
          'value' => $params['bonus_percentage'],
          'compare' => '>=',
          'type' => 'NUMERIC',
        ];
      }

      if ( isset ( $params['wagering'] ) && $params['wagering'] != '0' ) {
        $meta_query[] = [
          'key' => 'wagering',
          'value' => $params['wagering'],
          'compare' => '>=',
          'type' => 'NUMERIC',
        ];
      }

      if ( isset ( $params['casino'] ) && $params['casino'] ) {
        $meta_query[] = [
          'key'   => 'casino',
          'value' => $params['casino'],
        ];
      }

      if ( isset ( $params['casinos'] ) && $params['casinos'] ) {
        $meta_query[] = [
          'key'     => 'casino',
          'value'   => $params['casinos'],
          'compare' => 'IN'
        ];
      }
    }

    if ( is_singular( self::$post_type ) ) {
      if ( $post_in ) 
        $post_in = array_diff($post_in, [get_the_ID()]);
      else
        $args['post__not_in'] = [get_the_ID()];
    }

    if ( $post_in ) {
      $args['post__in'] = $post_in;
    }

    if ( $tax_query ) {
      $args['tax_query'] = $tax_query;
    }

    if ( $meta_query ) {
      $args['meta_query'] = $meta_query;
    }

    if ( isset( $params['filter'] ) && $params['filter'] && $params['filter_type'] === '2' ) {
      $args['posts_per_page'] = $total_posts;
    } else {
      $args['posts_per_page'] = ($total_posts > $page_posts && $page_posts > 0 || $total_posts < 0) ? $page_posts : $total_posts;
    }

    return $args;
  }

  public static function list_bonuses_module_args($params, $found_posts = null) {
    $args = [
      'list_type' => '3',
      'results_per_page' => $params['results_per_page'],
      'order_by' => $params['order_by'],
    ];

    if ( $params['filter'] && $params['filter_type'] === '2' ) {
      $args['number_of_results'] = $params['number_of_results'];
    } else {
      $args['number_of_results'] = -1;
    }

    if ( $found_posts ) {
      $args['found_posts'] = $found_posts;
    }

    return $args;
  }
}
