<?php 

class CF_Casinos_Args {

  private static $post_type = 'post';
  private static $subfix = '_listing';

  protected static function list_casino_taxonomies($params) {
    $tax_query = [];
    $taxonomies = [
      'casino-type' => 'casino_types' . self::$subfix,
      'bonus-type' => 'bonus_types' . self::$subfix,
      'provider' => 'software_providers' . self::$subfix,
      'payment' => 'payment_methods' . self::$subfix,
      'license' => 'licenses' . self::$subfix,
      'language' => 'languages' . self::$subfix,
      'currency' => 'currencies' . self::$subfix,
      'crypto-currency' => 'crypto_currencies' . self::$subfix,
      'platform' => 'platforms' . self::$subfix,
      'owner' => 'owners' . self::$subfix,
      'live-game' => 'live_games' . self::$subfix,
      'casino-status' => 'casino_status' . self::$subfix,
    ];

    foreach($taxonomies as $term => $taxonomy) :
      if (isset($params[$taxonomy]) && !empty($params[$taxonomy])) :
        if ( is_array( $params[$taxonomy] ) ) :
          foreach ($params[$taxonomy] as $term_id) {
            $tax_query[] = [
              'taxonomy' => $term,
              'field' => 'term_id',
              'terms' => $term_id,
            ];
          }
        else :
          $tax_query[] = [
            'taxonomy' => $term,
            'field' => 'term_id',
            'terms' => $params[$taxonomy],
          ];
        endif;
      endif;
    endforeach;

    return $tax_query;
  }

  protected static function bonuses_query($bonus, $min, $post_in) {
    global $wpdb;

    $geo_where = '';
    $geo_bonus = [];
    $found_geo_bonus = [];
    $default_bonus = [];
    $default_where = '';

    if ( $min ) {
      $default_where .= " AND meta_value >= ". $min;
    } 
    // if ( $max ) {
    //   $default_where .= " AND meta_value <= ". $max;
    // } 
    if ( $post_in ) $geo_where .= " AND M2.post_id IN (" . implode(',', $post_in) . ")";

    $select_geo = "SELECT M1.post_id, M1.meta_key as mk1, M2.meta_key as mk2, M2.meta_value FROM {$wpdb->prefix}postmeta M1, {$wpdb->prefix}postmeta M2 WHERE M1.post_id = M2.post_id AND M2.meta_key LIKE 'geo_targeted_bonuses_%_{$bonus}' AND M1.meta_key LIKE 'geo_targeted_bonuses_%_country' AND M1.meta_value = " . CF_Helpers_Common::get_session('country_id') . $geo_where;

    $geo_results = $wpdb->get_results($select_geo, ARRAY_A);

    if ( $geo_results ) :
      foreach ($geo_results as $result) :
        $key1 = str_replace('_country', '', $result['mk1']);
        $key2 = str_replace('_' . $bonus, '', $result['mk2']);
        
        if ( $post_in )
          $post_in = array_diff($post_in, [ $result['post_id'] ]);

        if ( $key1 === $key2 ) :
          $found_geo_bonus[] = $result['post_id'];

          // if ($result['meta_value'] >= $min && $result['meta_value'] <= $max) 
          if ( $result['meta_value'] >= $min ) 
            $geo_bonus[] = $result['post_id'];
        endif; 

      endforeach;
    endif;

    if ( $post_in ) $default_where .= " AND post_id IN (" . implode(',', $post_in) . ")";
    elseif ( $found_geo_bonus ) $default_where .= " AND post_id NOT IN (" . implode(',', $found_geo_bonus) . ")";

    $select_default = "SELECT post_id FROM {$wpdb->prefix}postmeta WHERE meta_key = 'default_bonuses_{$bonus}'" . $default_where;

    $default_results = $wpdb->get_results($select_default, OBJECT_K);

    if ( $default_results )  $default_bonus = array_keys($default_results);

    if( $geo_bonus && $default_bonus ) :
      return array_unique( array_merge( $geo_bonus, $default_bonus ) );
    elseif ( $geo_bonus ) :
      return $geo_bonus;
    elseif ( $default_bonus ) :
      return $default_bonus;
    endif;

    return false;
  }

  public static function list_casinos_args($params) {
    $args = [
      'post_type'   => self::$post_type,
      'post_status' => 'publish',
    ];
    $page_posts = (int)$params['results_per_page'];
    $total_posts = (int)$params['number_of_results'];
    $meta_query = isset( $params['meta_query'] ) ? $params['meta_query'] : [];
    $tax_query = isset( $params['tax_query'] ) ? $params['tax_query'] : [];
    $post_in =  isset( $params['found_posts'] ) ?  $params['found_posts'] : [];

    if ($params['list_type'] === '2') :
      $post_in = $params['casinos'];
      $args['orderby'] = $params['order_by_manually'];
    else :
      $args['orderby'] = $params['order_by']; 
    endif;

    if ($args['orderby'] === 'title') :
      $args['order'] = 'ASC';
    elseif ($args['orderby'] === 'meta_value_num') :
      $args['orderby'] = 'meta_value_num date';
      $args['meta_key'] = 'rating';
    else:
      $args['order'] = 'DESC';
    endif;

    if (isset($params['offset'])) 
      $args['offset'] = $params['offset'];
    
    if (isset($params['s'])) 
      $args['s'] = $params['s'];

    if ( isset($params['allowed_in']) && $params['allowed_in'] !== '0' ) :
      global $wpdb;
      $where = '';
      
      if ($params['allowed_in'] === '1') :
        $customer = CF_Helpers_Common::get_session(['geo_iso_code', 'country_id']);
        $where .= " AND m1.meta_value NOT LIKE '%". $customer['country_id'] ."%'";

        if ( $customer['geo_iso_code'] === 'US' ) :
          $us_page_id = CF_Helpers_Country::get_country_id_by_code('US', false);
          $where .= " AND m1.meta_value NOT LIKE '%{$us_page_id}%'";
        endif;
      elseif ($params['allowed_in'] === '2') :
        foreach ($params['specific_countries'] as $country) {
          $where .= " AND m1.meta_value NOT LIKE '%{$country}%'";
        }
      endif;

      if ( $post_in ) {
        $where .= " AND post_id IN (" . implode(',', $post_in) . ")";
      } 

      $select = "SELECT p.ID FROM {$wpdb->prefix}posts p LEFT JOIN {$wpdb->prefix}postmeta m1 ON m1.post_id = p.ID AND m1.meta_key = 'restricted' WHERE p.post_type = 'post' AND p.post_status = 'publish'" . $where;
      
      $results = $wpdb->get_results($select, OBJECT_K);

      if ( ! $results ) return false;

      $post_in = array_keys($results);
    endif;

    if ($params['list_type'] === '3') :
      if ($taxonomies = self::list_casino_taxonomies($params)) {
        $tax_query = array_merge($tax_query, $taxonomies);
      }

      if ( isset ( $params['rating'] ) && $params['rating'] != '0' ) {
        $meta_query[] = [
          'key' => 'rating',
          'value' => $params['rating'],
          'compare' => '>=',
          'type' => 'NUMERIC',
        ];
      }
      
      if ( isset ( $params['slots'] ) && $params['slots'] != '0' ) {
        $meta_query[] = [
          'key' => 'slots_amount',
          'value' => $params['slots'],
          'compare' => '>=',
          'type' => 'NUMERIC',
        ];
      }
      
      if ( isset ( $params['games'] ) && $params['games'] != '0' ) {
        $meta_query[] = [
          'key' => 'live_games_amount',
          'value' => $params['games'],
          'compare' => '>=',
          'type' => 'NUMERIC',
        ];
      }

      if ( isset ( $params['bonus_amount'] ) && $params['bonus_amount'] != '0' ) {
        if ( ! $post_in = self::bonuses_query('bonus_amount', $params['bonus_amount'], $post_in ) )  return false;
      }

      if ( isset ( $params['free_spins'] ) && $params['free_spins'] != '0' ) {
        if ( ! $post_in = self::bonuses_query('free_spins', $params['free_spins'], $post_in ) )  return false;
      }

      if ( isset ( $params['bonus_percentage'] ) && $params['bonus_percentage'] != '0' ) {
        if ( ! $post_in = self::bonuses_query('bonus_percentage', $params['bonus_percentage'], $post_in ) )  return false;
      }

      if ( isset ( $params['date'] ) && ( $params['date']['year'] !== '0' || $params['date']['month'] !== '0' ) ) {
        if ( $params['date']['year'] !== '0' && $params['date']['month'] === '0' ) {
          $meta_query[] = [
            'key' => 'year_founded',
            'value' => [$params['date']['year'] . '-01-01', $params['date']['year'] . '-12-31'],
            'compare' => 'BETWEEN',
            'type' => 'DATE',
          ];
        } elseif ( $params['date']['year'] === '0' && $params['date']['month'] !== '0' ) {
          $meta_query[] = [
            'key' => 'year_founded',
            'value' => '-'. $params['date']['month'] .'-',
            'compare' => 'LIKE',
            'type' => 'DATE',
          ];
        } else {
          $meta_query[] = [
            'key' => 'year_founded',
            'value' => [$params['date']['year'] . '-' . $params['date']['month'] . '-01', $params['date']['year'] . '-'. $params['date']['month'] .'-31'],
            'compare' => 'BETWEEN',
            'type' => 'DATE',
          ];
        }
      }
    endif;

    if ( is_singular( self::$post_type ) ) :
      if ( $post_in ) 
        $post_in = array_diff($post_in, [get_the_ID()]);
      else
        $args['post__not_in'] = [get_the_ID()];
    endif;

    if ( $post_in ) :
      $args['post__in'] = $post_in;
    endif;

    if ( $tax_query ) :
      $args['tax_query'] = $tax_query;
    endif;

    if ( $meta_query ) :
      $args['meta_query'] = $meta_query;
    endif;

    if ( isset( $params['filter'] ) && $params['filter'] && $params['filter_type'] === '2' ) :
      $args['posts_per_page'] = $total_posts;
    else :
      $args['posts_per_page'] = ($total_posts > $page_posts && $page_posts > 0 || $total_posts < 0) ? $page_posts : $total_posts;
    endif;

    return $args;
  }

  public static function list_casino_module_args($params, $found_posts = null) {
    $args = [
      'list_type' => '3',
      'results_per_page' => $params['results_per_page'],
      'order_by' => $params['order_by'],
      'allowed_in' => $params['allowed_in'],
    ];

    if ( $params['allowed_in'] === '2' ) {
      $args['specific_countries'] = $params['specific_countries'];
    }

    if ( $params['filter'] && $params['filter_type'] === '2' ) {
      $args['number_of_results'] = $params['number_of_results'];
    } else {
      $args['number_of_results'] = -1;
    }

    if ( $found_posts ) {
      $args['found_posts'] = $found_posts;
    }

    return $args;
  }
}
