<?php

class CF_Posttypes_Args {

  private static $subfix = '_listing';

  protected static function list_taxonomies($params) {
    $tax_query = [];
    $taxonomies = [
      'payment-category' => 'categories' . self::$subfix,
      'provider-feature' => 'features' . self::$subfix,
    ];

    foreach($taxonomies as $term => $taxonomy) :
      if (isset($params[$taxonomy]) && !empty($params[$taxonomy])) :
        if ( is_array( $params[$taxonomy] ) ) :
          foreach ($params[$taxonomy] as $term_id) {
            $tax_query[] = [
              'taxonomy' => $term,
              'field' => 'term_id',
              'terms' => $term_id,
            ];
          }
        else :
          $tax_query[] = [
            'taxonomy' => $term,
            'field' => 'term_id',
            'terms' => $params[$taxonomy],
          ];
        endif;
      endif;
    endforeach;

    return $tax_query;
  }

  public static function list_post_type_args($params) {
    $args = [
      'post_type' => $params['post_type'],
      'post_status' => 'publish',
    ];
    $page_posts  = (int)$params['results_per_page'];
    $total_posts = (int)$params['number_of_results'];
    $meta_query  = isset( $params['meta_query'] ) ? $params['meta_query'] : [];
    $tax_query   = isset( $params['tax_query'] ) ? $params['tax_query'] : [];
    $post_in     = isset( $params['found_posts'] ) ?  $params['found_posts'] : [];

    if ( ! isset( $params['order_by'] ) ) 
      $params['order_by'] = 'date';

    if ($params['list_type'] === '2') :
      $post_in = $params['selected'];
      $args['orderby'] = $params['order_by_manually'];
    else :
      $args['orderby'] = $params['order_by'];
      $args['post_parent'] = 0;
    endif;

    if ( $args['orderby'] === 'post__in' ) {
      $args['orderby'] = 'post__in';
    } elseif ( $args['orderby'] === 'title' ) {
      $args['orderby'] = 'title';
      $args['order'] = 'ASC';
    } elseif ( $args['orderby'] !== 'date' ) {
      $args['meta_key'] = $args['orderby'];
      $args['orderby'] = 'meta_value_num date';
    }

    if (isset($params['offset']))
      $args['offset'] = $params['offset'];

    if (isset($params['s']))
      $args['s'] = $params['s'];


    if ( ( isset($params['allowed_in'] ) && $params['allowed_in'] !== '0' ) ) :
      global $wpdb;
      $where = '';
      
      if ( $params['allowed_in'] === '1' ) :
        $customer = CF_Helpers_Common::get_session(['geo_iso_code', 'country_id']);
        $where .= " AND m1.meta_value NOT LIKE '%". $customer['country_id'] ."%'";

        if ( $customer['geo_iso_code'] === 'US' ) :
          $us_page_id = CF_Helpers_Country::get_country_id_by_code('US', false);
          $where .= " AND m1.meta_value NOT LIKE '%{$us_page_id}%'";
        endif;
      elseif ($params['allowed_in'] === '2') :
        foreach ($params['specific_countries'] as $country) {
          $where .= " AND m1.meta_value NOT LIKE '%{$country}%'";
        }
      endif;

      if ( $post_in ) {
        $where .= " AND post_id IN (" . implode(',', $post_in) . ")";
      } 

      $select = "SELECT p.ID FROM {$wpdb->prefix}posts p LEFT JOIN {$wpdb->prefix}postmeta m1 ON m1.post_id = p.ID AND m1.meta_key = 'restricted' WHERE p.post_type = '". $params['post_type'] . "' AND p.post_status = 'publish'" . $where;
      
      $results = $wpdb->get_results($select, OBJECT_K);

      if ( ! $results ) return false;

      $post_in = array_keys($results);
    endif;

    if ( isset($params['countries']) && $params['countries'] ) {
      $meta_query[] = [
        'key'     => 'countries',
        'value'   => CF_Helpers_Common::get_session('country_id'),
        'compare' => 'LIKE'
      ];
    }

    if ($params['list_type'] === '3') {
      if ($taxonomies = self::list_taxonomies($params)) {
        $tax_query = array_merge($tax_query, $taxonomies);
      }

      if ( isset($params['rating']) && $params['rating'] != '0' ) {
        $meta_query[] = [
          'key'     => 'rating',
          'value'   => $params['rating'],
          'compare' => '>=',
          'type' => 'NUMERIC',
        ];
      }

      if ( isset($params['number_of_games']) && $params['number_of_games'] != '0' ) {
        $meta_query[] = [
          'key'     => 'number_of_games',
          'value'   => $params['number_of_games'],
          'compare' => '>=',
          'type' => 'NUMERIC',
        ];
      }
    }

    if ( is_singular( $params['post_type'] ) ) :
      if ( $post_in )
        $post_in = array_diff($post_in, [get_the_ID()]);
      else
        $args['post__not_in'] = [get_the_ID()];
    endif;

    if ( $post_in ) :
      $args['post__in'] = $post_in;
    endif;

    if ( $tax_query ) :
      $args['tax_query'] = $tax_query;
    endif;

    if ( $meta_query ) :
      $args['meta_query'] = $meta_query;
    endif;

    $args['posts_per_page'] = ($total_posts > $page_posts && $page_posts > 0 || $total_posts < 0) ? $page_posts : $total_posts;

    return $args;
  }
}
