<?php 

class CF_Slots_Args {

  private static $post_type = 'slot';
  private static $subfix = '_listing';

  protected static function list_slots_taxonomies($params) {
    $tax_query = [];
    $taxonomies = [
      'provider' => 'providers' . self::$subfix,
      'slot-type' => 'types' . self::$subfix,
      'feature' => 'features' . self::$subfix,
      'theme' => 'themes' . self::$subfix,
    ];

    foreach($taxonomies as $term => $taxonomy) :
      if (isset($params[$taxonomy]) && !empty($params[$taxonomy])) :
        if ( is_array( $params[$taxonomy] ) ) :
          foreach ($params[$taxonomy] as $term_id) {
            $tax_query[] = [
              'taxonomy' => $term,
              'field' => 'term_id',
              'terms' => $term_id,
            ];
          }
        else :
          $tax_query[] = [
            'taxonomy' => $term,
            'field' => 'term_id',
            'terms' => $params[$taxonomy],
          ];
        endif;
      endif;
    endforeach;

    return $tax_query;
  }

  public static function list_slots_args($params) {
    $args = [
      'post_type' => 'slot',
      'post_status' => 'publish',
    ];
    $page_posts = (int) $params['results_per_page'];
    $total_posts = (int) $params['number_of_results'];
    $meta_query = isset( $params['meta_query'] ) ? $params['meta_query'] : [];
    $tax_query = isset( $params['tax_query'] ) ? $params['tax_query'] : [];
    $post_in =  isset( $params['found_posts'] ) ?  $params['found_posts'] : [];
    
    if ($params['list_type'] === '2') {
      $post_in = $params['slots'];
      $args['orderby'] = $params['order_by_manually'];
    } else {
      $args['orderby'] = $params['order_by']; 
    }

    if ( $args['orderby'] === 'post__in' ) {
      $args['orderby'] = 'post__in';
    } elseif ( $args['orderby'] !== 'date' ) {
      $args['meta_key'] = $args['orderby'];
      $args['orderby'] = 'meta_value_num date';
      $args['order'] = 'DESC';
    }

    if (isset($params['offset'])) 
      $args['offset'] = $params['offset'];
    
    if (isset($params['s'])) 
      $args['s'] = $params['s'];

    if ($params['list_type'] === '3') {
      if ($taxonomies = self::list_slots_taxonomies($params)) {
        $tax_query = array_merge($tax_query, $taxonomies);
      }

      if ( isset ( $params['rating'] ) && $params['rating'] != '0' ) {
        $meta_query[] = [
          'key' => 'rating',
          'value' => $params['rating'],
          'compare' => '>=',
          'type' => 'NUMERIC',
        ];
      }

      if ( isset ( $params['rtp'] ) && $params['rtp'] != '0' ) {
        $meta_query[] = [
          'key' => 'rtp',
          'value' => $params['rtp'],
          'compare' => '>=',
          'type' => 'NUMERIC',
        ];
      }

      if ( isset ( $params['min_bet'] ) && $params['min_bet'] != '0' ) {
        $meta_query[] = [
          'key' => 'minimum_bet',
          'value' => $params['min_bet'],
          'compare' => '>=',
          'type' => 'NUMERIC',
        ];
      }

      if ( isset ( $params['max_bet'] ) && $params['max_bet'] != '0' ) {
        $meta_query[] = [
          'key' => 'maximum_bet',
          'value' => $params['max_bet'],
          'compare' => '>=',
          'type' => 'NUMERIC',
        ];
      }

      if ( isset ( $params['max_win'] ) && $params['max_win'] != '0' ) {
        $meta_query[] = [
          'key' => 'maximum_win',
          'value' => $params['max_win'],
          'compare' => '>=',
          'type' => 'NUMERIC',
        ];
      }

      if ( isset ( $params['volatility'] ) && $params['volatility'] != '0' ) {
        $meta_query[] = [
          'key' => 'volatility',
          'value' => $params['volatility']
        ];
      }

      if ( isset ( $params['date'] ) && ( $params['date']['year'] !== '0' || $params['date']['month'] !== '0' ) ) {
        if ( $params['date']['year'] !== '0' && $params['date']['month'] === '0' ) {
          $meta_query[] = [
            'key' => 'date_launched',
            'value' => [$params['date']['year'] . '-01-01', $params['date']['year'] . '-12-31'],
            'compare' => 'BETWEEN',
            'type' => 'DATE',
          ];
        } elseif ( $params['date']['year'] === '0' && $params['date']['month'] !== '0' ) {
          $meta_query[] = [
            'key' => 'date_launched',
            'value' => '-'. $params['date']['month'] .'-',
            'compare' => 'LIKE',
            'type' => 'DATE',
          ];
        } else {
          $meta_query[] = [
            'key' => 'date_launched',
            'value' => [$params['date']['year'] . '-' . $params['date']['month'] . '-01', $params['date']['year'] . '-'. $params['date']['month'] .'-31'],
            'compare' => 'BETWEEN',
            'type' => 'DATE',
          ];
        }
      }
    }

    if ( is_singular( self::$post_type ) ) {
      if ( $post_in ) 
        $post_in = array_diff($post_in, [get_the_ID()]);
      else
        $args['post__not_in'] = [get_the_ID()];
    }

    if ( $post_in ) {
      $args['post__in'] = $post_in;
    }

    if ( $tax_query ) {
      $args['tax_query'] = $tax_query;
    }

    if ( $meta_query ) {
      $args['meta_query'] = $meta_query;
    }

    if ( isset( $params['filter'] ) && $params['filter'] && $params['filter_type'] === '2' ) {
      $args['posts_per_page'] = $total_posts;
    } else {
      $args['posts_per_page'] = ($total_posts > $page_posts && $page_posts > 0 || $total_posts < 0) ? $page_posts : $total_posts;
    }

    return $args;
  }

  public static function list_slots_module_args($params, $found_posts = null) {
    $args = [
      'list_type' => '3',
      'results_per_page' => $params['results_per_page'],
      'order_by' => $params['order_by'],
    ];
  
    if ( $params['filter'] && $params['filter_type'] === '2' ) {
      $args['number_of_results'] = $params['number_of_results'];
    } else {
      $args['number_of_results'] = -1;
    }
  
    if ( $found_posts ) {
      $args['found_posts'] = $found_posts;
    }
  
    return $args;
  }
}
