class MainMenu {
  body: HTMLBodyElement;
  mainMenuClassOpen = 'mmo';
  searchClassOpen = 'sfo';
  subMenuClassOpen = 'opened';
  btnMenu: HTMLBodyElement;
  btnSearch: HTMLBodyElement;
  mainMenuList: HTMLUListElement;
  subMenuLinks: HTMLLinkElement[];

  constructor() {
    this.body = document.querySelector('body');
    this.btnMenu = document.querySelector('.js-main-menu');
    this.btnSearch = document.querySelector('.js-btn-search');
    this.mainMenuList = document.querySelector('.primary-nav');
    if (this.mainMenuList) {
      this.subMenuLinks = [].slice.call(this.mainMenuList.querySelectorAll('.menu-item-has-children'));
      this.mainMenu();
      this.search();
      this.openSubMenu();
      this.closeTargetMainMenu();
    }
  }

  mainMenu() {
    if (this.btnMenu) {
      this.btnMenu.addEventListener('click', () => {
        if (!this.body.classList.contains(this.mainMenuClassOpen)) {
          this.btnMenu.classList.add('active');
          this.body.classList.add(this.mainMenuClassOpen);
        } else {
          this.closeMainMenu();
        }
      });
    }
  }

  search() {
    if (this.btnSearch) {
      this.btnSearch.addEventListener('click', () => {
        this.btnSearch.classList.toggle('active');
        this.body.classList.toggle(this.searchClassOpen);
      });
    }
  }
  
  closeMainMenu() {
    const openSubMenus = [].slice.call(this.mainMenuList.querySelectorAll('.' + this.subMenuClassOpen));
    this.body.classList.remove(this.mainMenuClassOpen);
    this.btnMenu.classList.remove('active');
    openSubMenus.forEach((openMenu: HTMLLIElement) => {
      openMenu.classList.remove(this.subMenuClassOpen);
    });
  }

  openSubMenu() {

    this.subMenuLinks.forEach(li => {
      li.addEventListener('click', (e) => {
        if (window.innerWidth < 1230) {
          var target = e.target as HTMLElement;
          if (target.tagName == 'LI') {
            if (li.classList.contains(this.subMenuClassOpen)) {
              li.classList.remove(this.subMenuClassOpen);
            } else {
              li.classList.add(this.subMenuClassOpen);
            }
          }
        }
      });
    });
  }

  closeTargetMainMenu() {
    document.addEventListener('click', (e) => {
      const target = e.target as HTMLElement;
      if (!target.classList.contains('primary-nav') && !target.closest('.primary-nav') && !target.closest('.js-main-menu')) {
        this.closeMainMenu();
      }
    });
  }
}

new MainMenu;