import Button from "../componets/_button";
import Common from "../componets/_common";
import HttpService from "../http/_httpSevices";
import { IListCasinoDatas } from "../interfaces/_intercase-datas";
import { CasinosResponse } from "../interfaces/_response";
import noUiSlider from 'nouislider';
import Choices from 'choices.js';

class ListCasinos {
  private BASE_URL = window.vars.ajax_url;
  http: HttpService;
  button: Button;
  common: Common;
  body: HTMLBodyElement;
  actions = {
    list_casinos: 'list_casinos',
    list_bonuses: 'list_bonuses',
  };
  datas: IListCasinoDatas[] = [];
  classes = {
    list_casinos          : '.js-list-casinos',
    append_casinos        : '.js-append-casinos',
    filter_form           : '.js-filter-form',
    filter_fields         : '.js-filter-change',
    filter_countrySwitch : '.js-country-switch',
    filter_range          : '.js-filter-range',
    filter_range_min      : '.js-range-min',
    filter_range_max      : '.js-range-max',
    filter_choice         : '.js-filter-choice',
    filter_open           : '.js-open-filter',
    count_casinos         : '.js-list-count',
    reset_btns            : '.js-reset-filter',
    loadmore              : '.js-casinos-loadmore',
    view_btns             : '.js-list-view',
    cta                   : '.js-list-cta',
    loader                : '.js-loader',
  };
  listAction     = [];
  listCasinos    = [];
  appendCasinos  = [];
  countrySwitch = [];
  choices        = [];
  choicesValues  = [];
  randerChoices  = [];
  ranges         = [];
  rangesValues   = [];
  btnsView       = [];
  btnsOpen       = [];
  btnsMore       = [];
  forms: HTMLFormElement[] = [];
  resetBtns      = [];
  changeMore     = [];
  loaders        = [];
  count_casinos  = [];
  clickCount: number[] = [];
  clickTypes: ('module' | 'args')[] = [];

  constructor() {
    this.http = new HttpService();
    this.button = new Button();
    this.common = new Common();
    this.body = document.querySelector('body');
    this.listCasinos = [].slice.call(document.querySelectorAll(this.classes.list_casinos));

    this.getDatas();
    this.onClickView();
    this.onClickLoadOpen();
    this.onClickLoadMore();
    this.onClickResetFilter();
  }

  getDatas() {
    if (this.listCasinos) {
      this.listCasinos.forEach((list: HTMLElement, i: number) => {
        this.setData(list, i);
      });
    }
  }

  getCasinos(data: { action: string, data: Record<string, unknown> }, i: number) {
    this.http.request("post", data).
      then((res) => {
        const data: CasinosResponse = JSON.parse(res.data);
        this.appendCasinos[i].insertAdjacentHTML('beforeend', data.html);
        this.loaders[i].style.display = 'none';
        if (data.more) {
          this.btnsMore[i].style.display = null;
        } else {
          const cta = this.listCasinos[i].querySelector(this.classes.cta);

          if (cta) {
            cta.style.display = null;
          }
        }

        if (this.count_casinos[i]) {
          this.setCountCasinos(this.count_casinos[i], data.found);
        }

        const btnsInfo = [].slice.call(this.listCasinos[i].querySelectorAll('.js-state')).slice(this.datas[i].args.offset * 2);
        this.button.states(btnsInfo);
        this.common.deferHTML(this.listCasinos[i].querySelectorAll('.js-defer-html'));
      })
      .catch((error) => {
        console.log(error);
      });
  }

  setData(list: HTMLElement, i: number) {
    this.datas.push({
      view: list.dataset.view,
      page_posts: parseInt(list.dataset.page_posts),
      total_posts: parseInt(list.dataset.total_posts),
      args: JSON.parse(list.dataset.args),
      ordered: !!list.dataset.ordered,
      module: JSON.parse(list.dataset.module)
    });
    this.listAction[i] = list.dataset.type === 'casinos' ? this.actions.list_casinos : this.actions.list_bonuses;
    this.appendCasinos[i] = list.querySelector(this.classes.append_casinos);
    this.btnsView[i] = [].slice.call(list.querySelectorAll(this.classes.view_btns));
    this.btnsOpen[i] = [].slice.call(list.querySelectorAll(this.classes.filter_open));
    this.btnsMore[i] = list.querySelector(this.classes.loadmore);
    this.loaders[i] = list.querySelector(this.classes.loader);

    this.ranges[i] = [].slice.call(list.querySelectorAll(this.classes.filter_range));
    this.rangesValues[i] = {
      min: [].slice.call(list.querySelectorAll(this.classes.filter_range_min)),
      // max: [].slice.call(list.querySelectorAll(this.classes.filter_range_max))
    };
    this.choices[i] = [].slice.call(list.querySelectorAll(this.classes.filter_choice));
    this.countrySwitch[i] = [].slice.call(list.querySelectorAll(this.classes.filter_countrySwitch));
    this.changeMore[i] = [].slice.call(list.querySelectorAll(this.classes.filter_fields));
    this.forms[i] = list.querySelector(this.classes.filter_form);
    this.resetBtns[i] = [].slice.call(list.querySelectorAll(this.classes.reset_btns));
    this.count_casinos[i] = [].slice.call(list.querySelectorAll(this.classes.count_casinos));

    this.setRanges(this.ranges[i], i);
    this.setChoices(this.choices[i], i);
    this.onChangeFilter(this.changeMore[i], i);
  }

  setClickLoadMore(btn: HTMLButtonElement, i: number) {
    if (btn) {
      this.clickTypes[i] = 'args';
      this.clickCount[i] = 0;

      btn.addEventListener('click', () => {
        this.clickCount[i]++;
        this.beforeGetCasinos(i);
      });
    }
  }

  setRanges(ranges, i:number ) {
    if (ranges) {
      ranges.forEach((range, j:number) => {
        noUiSlider.create(range, {
          start: [range.dataset.start],//[range.dataset.from, range.dataset.to],
          step: parseInt(range.dataset.step),
          connect: [false, true],
          range: {
            'min': [parseInt(range.dataset.from)],
            'max': [parseInt(range.dataset.to)]
          },
          format: {
            to: (value) => { return value },
            from: (value) => { return parseInt(value) }
          }
        }).on('slide', (values) => {
          const val: any = values;
          this.rangesValues[i]['min'][j].innerHTML = parseInt(val[0]);
          const input = this.listCasinos[i].querySelector(`input[name="${range.dataset.name}"]`);
          input.value = parseInt(val[0]);
          this.setChangeFilter(i);
            // this.rangesValues[i]['max'][j].innerHTML = parseInt(val[1]);
          });
      });
    }
  }

  setChoices(choices, i: number) {
    if (choices) {
      this.choicesValues[i] = [];
      this.randerChoices[i] = [];

      choices.forEach((choice, j: number) => {
        this.randerChoices[i].push(new Choices(choice, {
          removeItemButton: true,
          placeholder: true,
          searchFields: ['label'],
          searchFloor: 3,
          searchResultLimit: 5,
          addItems: false,
          itemSelectText: '',
          position: 'bottom',
        }));

        this.randerChoices[i][j].setChoices(async () => {
          try {
            const items = await fetch(this.BASE_URL + '?action=filter_terms&type=' + choice.dataset.type + '&selected=' + choice.dataset.selected + '&exclude=' + (choice.dataset.exclude ? choice.dataset.exclude : ''))
              .then(response => response.json())
              .then(data => data);
            this.choicesValues[i][j] = items;
            return items;
          } catch (err) {
            console.error(err);
          }
        });

        choice.addEventListener('change', () => {
          if (choice.name === 'specific_countries') {
            this.countrySwitch[i].map((item: HTMLInputElement) => {
              item.checked = false;
            });

            if (!choice.value) {
              const input = this.countrySwitch[i].find((item: HTMLInputElement) => !item.value);
              input.checked = true;
            }
          }
          this.setChangeFilter(i);
        }, false)
      });
    }
  }

  setChangeFilter(i: number) {
    const list = this.listCasinos[i];
    const cta: HTMLElement = list.querySelector(this.classes.cta);
    const form = this.forms[i];
    const formData = new FormData(form);
    const multiSelection = [
      'casino_types_listing',
      'bonus_types_listing',
      'bonus_occasion_listing',
      'live_games_listing',
      'casino_status_listing',
      'software_providers_listing',
      'licenses_listing',
      'languages_listing',
      'specific_countries',
      'currencies_listing',
      'platforms_listing',
      'owners_listing',
    ];

    this.appendCasinos[i].innerHTML = '';
    this.datas[i].module = JSON.parse(list.dataset.module);
    this.datas[i].module.allowed_in = '0';
    formData.forEach((value: string, key: string) => {
      if (value !== '0' && value !== '') {
        if (key === 'date') {
          const values = formData.getAll(key);
          this.datas[i].module['date'] = {
            month: values[0],
            year: values[1]
          }
        } else if (key === 'specific_countries') {
          this.datas[i].module.allowed_in = '2',
          this.datas[i].module[key] = formData.getAll(key);
        } else if (multiSelection.includes(key)) {
          this.datas[i].module[key] = formData.getAll(key);
        } else {
          this.datas[i].module[key] = value;
        }
      }
    });

    if (cta) {
      cta.style.display = 'none';
    }
    this.clickCount[i] = 0;
    this.clickTypes[i] = 'module';
    this.resetBtns[i].map((btn: HTMLButtonElement) => btn.style.display = null);
    this.beforeGetCasinos(i);
  }

  setResetFilter(items, i: number) {
    if (items) {
      items.forEach((btn: HTMLButtonElement) =>
        btn.addEventListener('click', () => {
          this.appendCasinos[i].innerHTML = '';
          this.resetFilter(i);
          this.beforeGetCasinos(i);
        })
      );
    }
  }

  setCountCasinos(counts, value) {
    if (counts) {
      counts.forEach(count => {
        count.innerHTML = value;
      });
    }
  }

  beforeGetCasinos(i) {
    const offset: number = this.datas[i].page_posts * this.clickCount[i];

    this.btnsMore[i].style.display = 'none';
    this.loaders[i].style.display = 'block';

    if ( offset + this.datas[i].page_posts  >= this.datas[i].total_posts && this.datas[i].total_posts > 0) {
      this.datas[i].args.posts_per_page = this.datas[i].total_posts - offset;
    }

    if (this.clickTypes[i] === 'args') {
      this.datas[i].args.offset = offset;
    }

    if (this.clickTypes[i] === 'module') {
      this.datas[i].module.offset = offset;
    }

    this.getCasinos({
      action: this.listAction[i],
      data: {
        args: this.clickTypes[i] === 'args' ? JSON.stringify(this.datas[i].args) : JSON.stringify(this.datas[i].module),
        type: this.clickTypes[i],
        view: this.datas[i].view,
        ordered: this.datas[i].ordered,
        total: this.clickTypes[i] === 'args' ? this.datas[i].total_posts : this.datas[i].module.number_of_results,
      }
    }, i);
  }

  resetFilter(i) {
    const list = this.listCasinos[i];
    const form = list.querySelector(this.classes.filter_form);
    const resetBtns = [].slice.call(list.querySelectorAll(this.classes.reset_btns));
    const cta = list.querySelector(this.classes.cta);

    this.datas[i].module = JSON.parse(list.dataset.module);
    this.clickCount[i] = 0;
    this.clickTypes[i] = 'args';
    form.reset();

    if (this.ranges[i]) {
      this.ranges[i].forEach((range, j:number) => {
        range.noUiSlider.set(range.dataset.start);
        this.rangesValues[i]['min'][j].innerHTML = parseInt(range.noUiSlider.get());
      });
    }

    if (this.randerChoices[i]) {
      this.randerChoices[i].forEach((choice, j: number) => {
        choice.clearStore();
        choice.setChoices(this.choicesValues[i][j]);
      });
    }

    if (cta) {
      cta.style.display = 'none';
    }
    resetBtns.forEach((btn: HTMLButtonElement) => btn.style.display = 'none');
  }

  onClickView() {
    if (this.btnsView) {
      this.btnsView.forEach((btns: HTMLButtonElement[], i:number) => {
        if (btns) {
          btns.forEach(btn => {
            btn.addEventListener('click', () => {
              this.appendCasinos[i].innerHTML = '';
              this.clickCount[i] = 0;
              this.datas[i].view = btn.dataset.view;
              this.beforeGetCasinos(i);
            })
          });
        }
      });
    }
  }

  onClickLoadOpen() {
    if (this.btnsOpen) {
      this.btnsOpen.forEach((btns: HTMLButtonElement[], i: number) => {
        if (btns) {
          btns.forEach(btn => {
            btn.addEventListener('click', () => {
              this.body.classList.toggle('of-d-sm-h');
              this.listCasinos[i].classList.toggle('filter-opened');
            });
          });
        }
      });
    }
  }

  onClickLoadMore() {

    if (this.btnsMore) {
      this.btnsMore.forEach((btn: HTMLButtonElement, i: number) => {
        this.setClickLoadMore(btn, i);
      });
    }
  }

  onChangeFilter(changeMore, i: number) {

    if (changeMore) {
      changeMore.forEach((item: HTMLFormElement) => {
        if (item.tagName === 'BUTTON') {
          item.addEventListener('click', () => {
            this.setChangeFilter(i);
          });
        }
        else if (item.name === 's') {
          item.addEventListener('keyup', () => {
            if (item.value.length > 2 || item.value == '') {
              this.setChangeFilter(i);
            }
          });
        }
        else {
          item.addEventListener('change', () => {
            if (this.randerChoices[i]) {
              this.randerChoices[i].forEach((choice, j: number) => {
                if (choice._baseId.includes(item.name)) {
                  choice.clearStore();
                  choice.setChoices(this.choicesValues[i][j]);
                }
              });
            }
            this.setChangeFilter(i);
          });
        }
      });
    }
  }

  onClickResetFilter() {
    if (this.resetBtns) {
      this.resetBtns.forEach((items, i: number) => {
        this.setResetFilter(items, i);
      })
    }
  }
}

new ListCasinos();
