import HttpService from "../http/_httpSevices";
import { IListNewsDatas } from "../interfaces/_intercase-datas";
import { IArgsConfig } from "../interfaces/_interface-query-args";
import { NewsResponse } from "../interfaces/_response";

class ListNews {
  http: HttpService;
  action = 'list_news';
  datas: IListNewsDatas[] = [];
  listNewsClass = '.js-list-news';
  listNews = [].slice.call(document.querySelectorAll('.js-list-news'));
  btnsMore = [];
  appendNews = [];
  loaders = [];
  clickCount: number[] = [];

  constructor() {
    this.http = new HttpService();
    this.getDatas();
    this.onClickLoad();
  }

  getDatas() {
    if (this.listNews) {
      this.listNews.forEach((list: HTMLElement) => {
        const args: IArgsConfig = JSON.parse(list.dataset.args);
        this.datas.push({
          page_posts: parseInt(list.dataset.page_posts),
          total_posts: parseInt(list.dataset.total_posts),
          args: args,
        })

        this.appendNews.push(list.querySelector('.js-append-news'));
        this.btnsMore.push(list.querySelector('.js-news-loadmore'));
        this.loaders.push(list.querySelector('.js-loader'));
      });
    }
  }

  getNews(data: { action: string, data: Record<string, unknown> }, i: number) {
    this.http.request("post", data).
      then((res) => {
        const data: NewsResponse = JSON.parse(res.data);
        this.appendNews[i].insertAdjacentHTML('beforeend', data.html);
        this.loaders[i].style.display = 'none';

        if (data.more) {
          this.btnsMore[i].style.display = null;
        } else {
          const cta = this.listNews[i].querySelector('.js-list-cta');
          
          if (cta) {
            cta.style.display = null;
          }
        }
      })
      .catch(error => {
        console.log(error);
      })
  }

  beforeGetNews(i) {
    const offset: number = this.datas[i].args.posts_per_page * this.clickCount[i];
    this.btnsMore[i].style.display = 'none';
    this.loaders[i].style.display = 'block';
    this.datas[i].args.offset = offset;

    if (2 * offset >= this.datas[i].total_posts && this.datas[i].total_posts > 0) {
      this.datas[i].args.posts_per_page = this.datas[i].total_posts - offset;
    }

    this.getNews({
      action: this.action,
      data: {
        args: JSON.stringify(this.datas[i].args),
        total: this.datas[i].total_posts
      }
    }, i)
  }

  onClickLoad() {

    if (this.btnsMore) {
      this.btnsMore.forEach((btn: HTMLButtonElement, i: number) => {
        this.clickCount.push(0);

        btn.addEventListener('click', () => {
          this.clickCount[i]++;
          this.beforeGetNews(i);
        });
      });
    }
  }
}

new ListNews();