import Button from "../componets/_button";
import Common from "../componets/_common";
import HttpService from "../http/_httpSevices";
import { IListPostType } from "../interfaces/_intercase-datas";
import { IArgsConfig, IPostTypes } from "../interfaces/_interface-query-args";
import { PostTypeResponse } from "../interfaces/_response";
import noUiSlider from 'nouislider';
import Choices from 'choices.js';

class ListPostType {
  private BASE_URL = window.vars.ajax_url;
  http: HttpService;
  button: Button;
  common: Common;
  body: HTMLBodyElement;
  datas: IListPostType[] = [];
  actions = {
    list_post_type: 'list_post_type',
    count_casinos: 'count_casinos',
  };
  classes = {
    list           : '.js-list-post-type',
    append         : '.js-append-results',
    form           : '.js-filter-form',
    fields         : '.js-filter-change',
    country_switch : '.js-country-switch',
    change         : '.js-filter-change',
    range          : '.js-filter-range',
    range_min      : '.js-range-min',
    range_max      : '.js-range-max',
    choice         : '.js-filter-choice',
    total          : '.js-total',
    count          : '.js-list-count',
    reset          : '.js-reset-filter',
    open           : '.js-open-filter',
    loadMore       : '.js-list-loadmore',
    cta            : '.js-list-cta',
    loader         : '.js-loader'
  }
  lists          = [];
  appends        = [];
  forms          = [];
  changes        = [];
  countrySwitch  = [];
  ranges         = [];
  rangesValues   = [];
  choices        = [];
  choicesValues  = [];
  randerChoices  = [];
  counts         = [];
  loadMoreBtns   = [];
  resets         = [];
  opens          = [];
  loaders        = [];
  clickCount : number[] = [];
  clickTypes: ('module' | 'args')[] = [];
  initialModule: IPostTypes;

  constructor() {
    this.onInit();
  }

  onInit() {
    this.http = new HttpService();
    this.button = new Button();
    this.common = new Common();
    this.body = document.querySelector('body');
    this.lists = [].slice.call(document.querySelectorAll(this.classes.list));
    this.getDatas();
    this.rangesInit();
    this.choicesInit();
    this.onClickLoadMore();
    this.onChangeFilter();
    this.onClickResetFilter();
    this.onClickLoadOpen();
  }

  getDatas() {
    if (this.lists) {
      this.lists.map((list, i) => {
        this.pushData(list);
        this.getTotalCasinos(this.datas[i].post_ids, this.datas[i].post_type, list);
      });
    }
  }

  pushData(list: HTMLElement) {
    const args: IArgsConfig = JSON.parse(list.dataset.args);
    this.initialModule = {
      post_type: list.dataset.type,
      post_status: 'publish',
      list_type: '3',
      offset: 0,
      order_by: args.meta_key,
      number_of_results: -1,
      results_per_page: parseInt(list.dataset.page_posts)
    }

    this.datas.push({
      page_posts: parseInt(list.dataset.page_posts),
      total_posts: parseInt(list.dataset.total_posts),
      args: args,
      module: this.initialModule,
      post_type: list.dataset.type,
      post_ids: list.dataset.posts
    });

    this.appends.push(list.querySelector(this.classes.append));
    this.forms.push(list.querySelector(this.classes.form));
    this.changes.push([].slice.call(list.querySelectorAll(this.classes.change)));
    this.countrySwitch.push([].slice.call(list.querySelectorAll(this.classes.country_switch)));
    this.ranges.push([].slice.call(list.querySelectorAll(this.classes.range)));
    this.choices.push([].slice.call(list.querySelectorAll(this.classes.choice)));
    this.rangesValues.push([].slice.call(list.querySelectorAll(this.classes.range_min)));
    this.counts.push([].slice.call(list.querySelectorAll(this.classes.count)));
    this.resets.push([].slice.call(list.querySelectorAll(this.classes.reset)));
    this.opens.push([].slice.call(list.querySelectorAll(this.classes.open)));
    this.loadMoreBtns.push(list.querySelector(this.classes.loadMore));
    this.loaders.push(list.querySelector(this.classes.loader));
    this.clickTypes.push('args');
    this.clickCount.push(0);
  }

  rangesInit() {
    if (this.ranges) {
      this.ranges.map((ranges, i: number) => {
        ranges.map((range, j: number) => {
          noUiSlider.create(range, {
            start: [range.dataset.start],
            step: parseInt(range.dataset.step),
            connect: [false, true],
            range: {
              'min': [parseInt(range.dataset.from)],
              'max': [parseInt(range.dataset.to)]
            },
            format: {
              to: (value) => { return value },
              from: (value) => { return parseInt(value) }
            }
          }).on('slide', (values) => {
            const val: any = values;
            this.rangesValues[i][j].innerHTML = parseInt(val[0]);
            const input = this.lists[i].querySelector(`input[name="${range.dataset.name}"]`);
            input.value = parseInt(val[0]);
            this.changeFilter(i);
          });
        });
      });
    }
  }

  choicesInit() {
    if (this.choices) {
      this.choices.map((choices: any[], i: number) => {
        this.choicesValues[i] = [];
        this.randerChoices[i] = [];

        choices.map((choice, j: number) => {
          this.randerChoices[i].push(new Choices(choice, {
            removeItemButton: true,
            placeholder: true,
            searchFields: ['label'],
            searchFloor: 3,
            searchResultLimit: 5,
            addItems: false,
            itemSelectText: '',
            position: 'bottom',
          }));

          this.randerChoices[i][j].setChoices(async () => {
            try {
              const items = await fetch(this.BASE_URL + '?action=filter_terms&type=' + choice.dataset.type + '&selected=' + choice.dataset.selected)
                .then(response => response.json())
                .then(data => data);
              this.choicesValues[i][j] = items;
              return items;
            } catch (err) {
              console.error(err);
            }
          });

          choice.addEventListener('change', () => {
            if (choice.name === 'specific_countries') {
              this.countrySwitch[i].map((item: HTMLInputElement) => {
                item.checked = false;
              });
  
              if (!choice.value) {
                const input = this.countrySwitch[i].find((item: HTMLInputElement) => !item.value);

                if (input) {
                  input.checked = true;
                }
              }
            }
            this.changeFilter(i);
          }, false)
        });
      });

    }
  }

  beforeGetPosts(i) {
    const offset: number = this.datas[i].page_posts * this.clickCount[i];

    this.loadMoreBtns[i].style.display = 'none';
    this.loaders[i].style.display = 'block';

    if ( offset + this.datas[i].page_posts >= this.datas[i].total_posts && this.datas[i].total_posts > 0) {
      this.datas[i].args.posts_per_page = this.datas[i].total_posts - offset;
    }

    if (this.clickTypes[i] === 'args') {
      this.datas[i].args.offset = offset;
    }

    if (this.clickTypes[i] === 'module') {
      this.datas[i].module.offset = offset;
    }

    this.getPosts({
      action: this.actions.list_post_type,
      data: {
        args: this.clickTypes[i] === 'args' ? JSON.stringify(this.datas[i].args) : JSON.stringify(this.datas[i].module),
        type: this.clickTypes[i],
        total: this.clickTypes[i] === 'args' ? this.datas[i].total_posts : this.datas[i].module.number_of_results,
        post_type: this.datas[i].post_type
      }
    }, i)
  }

  getPosts(data: {
    action: string, data: {
      args      : string,
      type      : string,
      total     : number,
      post_type : string
    }
  }, i: number) {
    this.http.request("post", data).
      then((res) => {
        const data: PostTypeResponse = JSON.parse(res.data);
        this.appends[i].insertAdjacentHTML('beforeend', data.html);
        this.loaders[i].style.display = 'none';
        this.getTotalCasinos(data.posts, this.datas[i].post_type, this.lists[i]);
        const btnsInfo = [].slice.call(this.lists[i].querySelectorAll('.js-state')).slice(this.datas[i].args.offset * 2);
        this.button.states(btnsInfo);
        this.common.deferHTML(this.lists[i].querySelectorAll('.js-defer-html'));

        if (data.more) {
          this.loadMoreBtns[i].style.display = null;
        } else {
          const cta = this.lists[i].querySelector(this.classes.cta);

          if (cta) {
            cta.style.display = null;
          }
        }

        if (this.counts[i]) {
          this.counts[i].map((count: HTMLElement) => {
            count.textContent = data.found;
          });
        }
      })
      .catch(error => {
        console.log(error);
      });
  }

  async getTotalCasinos(posts: string, post_type: string, list: HTMLElement) {
    if (posts) {
      const reuslts = await fetch(this.BASE_URL + '?action='+ this.actions.count_casinos +'&posts=' + posts + '&type=' + post_type)
        .then(response => response.json())
        .then(data => data);
      for (const data in reuslts) {
        const items = [].slice.call(list.querySelectorAll(`.lpt-${data} ${this.classes.total}`));
        this.updateCountCasinos(items, reuslts[data])
      }
    }
    // this.http.request('get', {
    //   action: this.actions.count_casinos,
    //   posts: posts,
    //   type: post_type
    // }).then(res => {
    //   const reuslts = JSON.parse(res.data);
    //   for (const data in reuslts) {
    //     const items = [].slice.call(list.querySelectorAll(`.lpt-${data} ${this.classes.total}`));
    //     this.updateCountCasinos(items, reuslts[data])
    //   }
    // }).catch (error => console.log(error));
  }

  updateCountCasinos(items: HTMLElement[], value: string) {
    items.map((item: HTMLElement) => {
      item.textContent = value;
    });
  }

  onClickLoadMore() {

    if (this.loadMoreBtns) {
      this.loadMoreBtns.map((btn, i) => {
        this.clickLoadMore(btn, i);
      });
    }
  }

  clickLoadMore(btn: HTMLButtonElement, i: number) {

    btn.addEventListener('click', () => {
      this.clickCount[i]++;
      this.beforeGetPosts(i);
    });
  }

  onChangeFilter() {
    if (this.changes) {
      this.changes.map((items, i: number) => {
        items.map(item => {
          if (item.tagName === 'BUTTON') {
            item.addEventListener('click', () => {
              this.changeFilter(i);
            });
          }
          else if (item.name === 's') {
            item.addEventListener('keyup', () => {
              this.changeFilter(i);
            });
          }
          else {
            item.addEventListener('change', () => {
              if (this.randerChoices[i]) {
                this.randerChoices[i].forEach((choice, j: number) => {
                  if (choice._baseId.includes(item.name)) {
                    choice.clearStore();
                    choice.setChoices(this.choicesValues[i][j]);
                  }
                });
              }
              this.changeFilter(i);
            });
          }
        });
      });
    }
  }

  changeFilter(i: number) {
    const list = this.lists[i];
    const cta: HTMLElement = list.querySelector(this.classes.cta);
    const form = this.forms[i];
    const formData = new FormData(form);
    const multiSelection = [
      'features_listing',
    ];

    this.appends[i].innerHTML = '';
    this.datas[i].module = { ...this.initialModule };
    if (!formData.has('specific_countries')) {
      this.datas[i].module.allowed_in = '0';
    }
    formData.forEach((value: string, key: string) => {
      if (value !== '0' && value !== '') {
        if (key === 'date') {
          const values = formData.getAll(key);
          this.datas[i].module['date'] = {
            month: values[0],
            year: values[1]
          }
        } else if (key === 'specific_countries') {
          this.datas[i].module.allowed_in = '2';
          this.datas[i].module[key] = formData.getAll(key);
        } else if (multiSelection.includes(key)) {
          this.datas[i].module[key] = formData.getAll(key);
        } else {
          this.datas[i].module[key] = value;
        }
      }
    });

    if (cta) {
      cta.style.display = 'none';
    }
    this.clickCount[i] = 0;
    this.clickTypes[i] = 'module';
    this.resets[i].map((btn: HTMLButtonElement) => btn.style.display = null);
    this.beforeGetPosts(i);
  }

  onClickResetFilter() {
    if (this.resets) {
      this.resets.map((resets: HTMLButtonElement[], i: number) => {
        resets.map((reset: HTMLButtonElement) => {
          this.resetFilter(reset, i);
        });
      })
    }
  }

  resetFilter(reset: HTMLElement, i: number) {
    reset.addEventListener('click', () => {
      const list = this.lists[i];
      const form = list.querySelector(this.classes.form);
      const resetBtns = [].slice.call(list.querySelectorAll(this.classes.reset));
      const cta = list.querySelector(this.classes.cta);

      this.appends[i].innerHTML = '';
      this.datas[i].module = this.initialModule;
      this.clickCount[i] = 0;
      this.clickTypes[i] = 'args';
      form.reset();

      if (this.ranges[i]) {
        this.ranges[i].map((range, j:number) => {
          range.noUiSlider.set(range.dataset.start);
          this.rangesValues[i][j].innerHTML = parseInt(range.noUiSlider.get());
        });
      }

      if (this.randerChoices[i]) {
        this.randerChoices[i].forEach((choice, j: number) => {
          choice.clearStore();
          choice.setChoices(this.choicesValues[i][j]);
        });
      }

      if (cta) {
        cta.style.display = 'none';
      }

      resetBtns.map((btn: HTMLButtonElement) => btn.style.display = 'none');
      this.beforeGetPosts(i);
    });
  }

  onClickLoadOpen() {
    if (this.opens) {
      this.opens.forEach((opens: HTMLButtonElement[], i: number) => {
        opens.map((open: HTMLButtonElement) => {
          open.addEventListener('click', () => {
            this.body.classList.toggle('of-d-sm-h');
            this.lists[i].classList.toggle('filter-opened');
          });
        });
      });
    }
  }
}

new ListPostType();
