import Button from "../componets/_button";
import Common from "../componets/_common";
import HttpService from "../http/_httpSevices";
import { IListSlotsDatas } from "../interfaces/_intercase-datas";
import { SlotsResponse } from "../interfaces/_response";
import noUiSlider from 'nouislider';
import Choices from 'choices.js';

class ListSlots {
  private BASE_URL = window.vars.ajax_url;
  http: HttpService;
  button: Button;
  common: Common;
  body: HTMLBodyElement;
  action = 'list_slots';
  datas: IListSlotsDatas[] = [];
  classes = {
    list      : '.js-list-slots',
    append    : '.js-append-slots',
    form      : '.js-filter-form',
    change    : '.js-filter-change',
    range     : '.js-filter-range',
    range_min : '.js-range-min',
    range_max : '.js-range-max',
    choice    : '.js-filter-choice',
    reset     : '.js-reset-filter',
    open      : '.js-open-filter',
    count     : '.js-list-count',
    loadMore  : '.js-slots-loadmore',
    cta       : '.js-list-cta',
    loader    : '.js-loader'
  }
  listSlots     = []
  appendSlots   = [];
  forms         = [];
  changes       = [];
  ranges        = [];
  rangesValues  = [];
  choices       = [];
  choicesValues = [];
  randerChoices = [];
  loadMoreBtns  = [];
  resets        = [];
  opens         = [];
  counts        = [];
  loaders       = [];
  clickCount: number[] = [];
  clickTypes: ('module' | 'args')[] = [];

  constructor() {
    this.http = new HttpService();
    this.button = new Button();
    this.common = new Common();
    this.body = document.querySelector('body');
    this.listSlots = [].slice.call(document.querySelectorAll(this.classes.list));
    this.getDatas();
  }

  getDatas() {
    if (this.listSlots) {
      this.listSlots.map((list: HTMLElement) => {
        this.pushData(list);
      });

      this.rangesInit();
      this.choicesInit();
      this.onClickLoadMore();
      this.onChangeFilter();
      this.onClickResetFilter();
      this.onClickLoadOpen();
    }
  }

  pushData(list: HTMLElement) {
    this.datas.push({
      page_posts: parseInt(list.dataset.page_posts),
      total_posts: parseInt(list.dataset.total_posts),
      args: JSON.parse(list.dataset.args),
      module: JSON.parse(list.dataset.module)
    });

    this.appendSlots.push(list.querySelector(this.classes.append));
    this.forms.push(list.querySelector(this.classes.form));
    this.changes.push([].slice.call(list.querySelectorAll(this.classes.change)));
    this.ranges.push([].slice.call(list.querySelectorAll(this.classes.range)));
    this.rangesValues.push([].slice.call(list.querySelectorAll(this.classes.range_min)));
    this.choices.push([].slice.call(list.querySelectorAll(this.classes.choice)));
    this.counts.push([].slice.call(list.querySelectorAll(this.classes.count)));
    this.resets.push([].slice.call(list.querySelectorAll(this.classes.reset)));
    this.opens.push([].slice.call(list.querySelectorAll(this.classes.open)));
    this.loadMoreBtns.push(list.querySelector(this.classes.loadMore));
    this.loaders.push(list.querySelector(this.classes.loader));
    this.clickTypes.push('args');
    this.clickCount.push(0);
  }

  rangesInit() {
    if (this.ranges) {
      this.ranges.map((ranges, i: number) => {
        ranges.map((range, j: number) => {
          noUiSlider.create(range, {
            start: [range.dataset.start],//[range.dataset.from, range.dataset.to],
            step: parseInt(range.dataset.step),
            connect: [false, true],
            range: {
              'min': [parseInt(range.dataset.from)],
              'max': [parseInt(range.dataset.to)]
            },
            format: {
              to: (value) => { return value },
              from: (value) => { return parseInt(value) }
            }
          }).on('slide', (values) => {
            const val: any = values;
            this.rangesValues[i][j].innerHTML = parseInt(val[0]);
            const input = this.listSlots[i].querySelector(`input[name="${range.dataset.name}"]`);
            input.value = parseInt(val[0]);
            this.changeFilter(i);
          });
        });
      });
    }
  }

  choicesInit() {
    if (this.choices) {
      this.choices.map((choices: HTMLElement[], i: number) => {
        this.choicesValues[i] = [];
        this.randerChoices[i] = [];

        choices.map((choice, j: number) => {
          this.randerChoices[i].push(new Choices(choice, {
            removeItemButton: true,
            placeholder: true,
            searchFields: ['label'],
            searchFloor: 3,
            searchResultLimit: 5,
            addItems: false,
            itemSelectText: '',
            position: 'bottom',
          }));

          this.randerChoices[i][j].setChoices(async () => {
            try {
              const items = await fetch(this.BASE_URL + '?action=filter_terms&type=' + choice.dataset.type + '&selected=' + choice.dataset.selected)
                .then(response => response.json())
                .then(data => data);
              this.choicesValues[i][j] = items;
              return items;
            } catch (err) {
              console.error(err);
            }
          });

          choice.addEventListener('change', () => {
            this.changeFilter(i);
          }, false)
        });
      });

    }
  }

  beforeGetSlots(i) {
    const offset: number = this.datas[i].page_posts * this.clickCount[i];

    this.loadMoreBtns[i].style.display = 'none';
    this.loaders[i].style.display = 'block';

    if ( offset + this.datas[i].page_posts >= this.datas[i].total_posts && this.datas[i].total_posts > 0) {
      this.datas[i].args.posts_per_page = this.datas[i].total_posts - offset;
    }

    if (this.clickTypes[i] === 'args') {
      this.datas[i].args.offset = offset;
    }

    if (this.clickTypes[i] === 'module') {
      this.datas[i].module.offset = offset;
    }

    this.getSlots({
      action: this.action,
      data: {
        args: this.clickTypes[i] === 'args' ? JSON.stringify(this.datas[i].args) : JSON.stringify(this.datas[i].module),
        type: this.clickTypes[i],
        total: this.clickTypes[i] === 'args' ? this.datas[i].total_posts : this.datas[i].module.number_of_results,
      }
    }, i)
  }

  getSlots(data: { action: string, data: {args: string, type: string, total: number} }, i: number) {
    this.http.request("post", data).
      then((res) => {
        const data: SlotsResponse = JSON.parse(res.data);
        this.appendSlots[i].insertAdjacentHTML('beforeend', data.html);
        this.loaders[i].style.display = 'none';
        const btnsInfo = [].slice.call(this.listSlots[i].querySelectorAll('.js-state')).slice(this.datas[i].args.offset * 2);
        this.button.states(btnsInfo);
        this.common.deferHTML(this.listSlots[i].querySelectorAll('.js-defer-html'));

        if (data.more) {
          this.loadMoreBtns[i].style.display = null;
        } else {
          const cta = this.listSlots[i].querySelector(this.classes.cta);

          if (cta) {
            cta.style.display = null;
          }
        }

        if (this.counts[i]) {
          this.counts[i].map(count => {
            count.innerHTML = data.found;
          });
        }
      })
      .catch(error => {
        console.log(error);
      });
  }

  onClickLoadMore() {

    if (this.loadMoreBtns) {
      this.loadMoreBtns.map((btn: HTMLButtonElement, i: number) => {
        this.clickLoadMore(btn, i);
      });
    }
  }

  clickLoadMore(btn: HTMLButtonElement, i: number) {

    btn.addEventListener('click', () => {
      this.clickCount[i]++;
      this.beforeGetSlots(i);
    });
  }

  onChangeFilter() {
    if (this.changes) {
      this.changes.map((items: HTMLFormElement[], i: number) => {
        items.map((item: HTMLFormElement) => {
          if (item.tagName === 'BUTTON') {
            item.addEventListener('click', () => {
              this.changeFilter(i);
            });
          }
          else if (item.name === 's') {
            item.addEventListener('keyup', () => {
              if (item.value.length > 2 || item.value == '') {
                this.changeFilter(i);
              }
            });
          }
          else {
            item.addEventListener('change', () => {
              this.changeFilter(i);
            });
          }
        });
      });
    }
  }

  changeFilter(i: number) {
    const list = this.listSlots[i];
    const cta: HTMLElement = list.querySelector(this.classes.cta);
    const form = this.forms[i];
    const formData = new FormData(form);
    const multiSelection = [
      'features_listing',
      'themes_listing',
      'providers_listing'
    ];

    this.appendSlots[i].innerHTML = '';
    this.datas[i].module = JSON.parse(list.dataset.module);

    formData.forEach((value: string, key: string) => {
      if (value !== '0' && value !== '') {
        if (key === 'date') {
          const values = formData.getAll(key);
          this.datas[i].module['date'] = {
            month: values[0],
            year: values[1]
          }
        } else if (multiSelection.includes(key)) {
          this.datas[i].module[key] = formData.getAll(key);
        } else {
          this.datas[i].module[key] = value;
        }
      }
    });

    if (cta) {
      cta.style.display = 'none';
    }
    this.clickCount[i] = 0;
    this.clickTypes[i] = 'module';
    this.resets[i].map((btn: HTMLButtonElement) => btn.style.display = null);
    this.beforeGetSlots(i);
  }

  onClickResetFilter() {
    if (this.resets) {
      this.resets.map((resets: HTMLButtonElement[], i: number) => {
        resets.map((reset: HTMLButtonElement) => {
          this.resetFilter(reset, i);
        });
      })
    }
  }

  resetFilter(reset: HTMLElement, i: number) {
    reset.addEventListener('click', () => {
      const list = this.listSlots[i];
      const form = list.querySelector(this.classes.form);
      const resetBtns = [].slice.call(list.querySelectorAll(this.classes.reset));
      const cta = list.querySelector(this.classes.cta);

      this.appendSlots[i].innerHTML = '';
      this.datas[i].module = JSON.parse(list.dataset.module);
      this.clickCount[i] = 0;
      this.clickTypes[i] = 'args';
      form.reset();

      if (this.ranges[i]) {
        this.ranges[i].map((range, j:number) => {
          range.noUiSlider.set(range.dataset.start);
          this.rangesValues[i][j].innerHTML = parseInt(range.noUiSlider.get());
        });
      }

      if (this.randerChoices[i]) {
        this.randerChoices[i].map((choice, j: number) => {
          choice.clearStore();
          choice.setChoices(this.choicesValues[i][j]);
        });
      }

      if (cta) {
        cta.style.display = 'none';
      }
      resetBtns.map((btn: HTMLButtonElement) => btn.style.display = 'none');
      this.beforeGetSlots(i);
    });
  }

  onClickLoadOpen() {
    if (this.opens) {
      this.opens.forEach((opens: HTMLButtonElement[], i: number) => {
        opens.map((open: HTMLButtonElement) => {
          open.addEventListener('click', () => {
            this.body.classList.toggle('of-d-sm-h');
            this.listSlots[i].classList.toggle('filter-opened');
          });
        });
      });
    }
  }
}

new ListSlots();
